/* dis9830: a portable disassembler for the HP 9830A/B calculator */
/* Copyright (c) 2013-2014, Robert Ferguson (rob@bitscience.ca) */
/* All rights reserved; see LICENSE for details */

#include <stdio.h>
#include <stdlib.h>

typedef struct _node {
	int n;
	union {
		struct _node *child[2];
		int opcode;
	};
} Node;


typedef struct _symbol {
	char *name;
	unsigned short address;
	int flag;
	unsigned short reference;
	struct _symbol *next;
} Symbol;


typedef struct _inst {
	unsigned short v;
	unsigned short address;
	int opcode;
	int flag;
	char r, g;
	unsigned short n;
	int mod;
	Symbol *s;
	struct _inst *next;
} Inst;

#define INST_SYM	0x1
#define INST_MEM	0x2
#define INST_SHIFT	0x4
#define INST_BRANCH	0x8
#define INST_DEV	0x10
#define INST_DATA	0x20

#define SYM_FORCE	0x1
#define SYM_DATA	0x2

#ifdef DEBUG
extern void dumptree(Node *t);
#endif

extern Node *insert(Node *t, char *bits, int i, int p, int opcode);
extern int match(Node *t, unsigned short v);

extern Inst *decode(int opcode, unsigned short v, unsigned short address);
extern void queueinst(Inst *i);
extern Inst *findinst(int address);
extern void dumpinst(int verbose);

extern Symbol *addsymbol(char *name, unsigned short address);
extern Symbol *findsymbol(unsigned short address);
extern void dumpsymbol(void);

extern void error(char *s, ...);

/* convenience macros for doing page-relative address ops */

#define PAGEINDEX(x)	((x) >> 11)
#define PAGEOFFSET(x)	((x) & ((1<<11) - 1))

#define FMT_HEADER 	0x1
#define FMT_XNS		0x2
